package org.artifactory.projects;

import lombok.NonNull;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.Diff;
import org.apache.commons.lang3.builder.DiffResult;
import org.artifactory.config.CentralConfigDiffValidator;
import org.artifactory.config.CentralConfigKey;
import org.jfrog.common.config.diff.DiffUtils;
import org.springframework.stereotype.Component;

import java.util.Set;

/**
 * @author BarakH
 */
@Component
public class ProjectAdminValidatorImpl implements CentralConfigDiffValidator {

    private final Set<String> approvedConfigChangesForProjectAdmin = Set
            .of(CentralConfigKey.localRepositoriesMap.getKey(),
                    CentralConfigKey.remoteRepositoriesMap.getKey(),
                    CentralConfigKey.virtualRepositoriesMap.getKey(),
                    CentralConfigKey.distributionRepositoriesMap.getKey(),
                    CentralConfigKey.releaseBundlesRepositoriesMap.getKey(),
                    "repoLayouts");

    /**
     * Don't call it with admin users. Make your authorization check before you are calling it.
     */
    @Override
    public void verifyConfigChangesAllowed(DiffResult diffResult) throws IllegalArgumentException {
        for (Diff<?> diff : diffResult.getDiffs()) {
            String field = getFieldName(diff);
            if (!approvedConfigChangesForProjectAdmin.contains(field)) {
                throw new IllegalArgumentException(
                        "Project admin is not allowed to make those Artifactory configuration changes.");
            }
        }
    }

    @NonNull
    private String getFieldName(Diff<?> diff) {
        String field = diff.getFieldName();
        if (StringUtils.isEmpty(field)) {
            return StringUtils.EMPTY;
        }
        if (field.contains(DiffUtils.DELIMITER)) {
            field = field.substring(0, field.indexOf(DiffUtils.DELIMITER));
        }
        return field;
    }
}
