package org.artifactory.storage;


import lombok.Data;
import org.artifactory.api.repo.storage.RepoStorageSummaryInfo;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author BarakH
 */
@Data
public class StorageUsage {

    long revision;
    private final Map<String, Long> repoStorageSum;
    private final Map<String, Long> projectsStorageSum;

    public StorageUsage() {
        revision = 1;
        repoStorageSum = new HashMap<>();
        projectsStorageSum = new HashMap<>();
    }

    public StorageUsage(StorageSummaryInfo info) {
        this.revision = 1;
        this.projectsStorageSum = info.getProjectsTotalSizes();
        this.repoStorageSum = info.getRepoStorageSummaries().values().stream()
                .collect(Collectors.toMap(RepoStorageSummaryInfo::getRepoKey, RepoStorageSummaryInfo::getUsedSpace));
    }

    public StorageUsage(StorageUsage other) {
        revision = other.revision;
        repoStorageSum = new HashMap<>(other.repoStorageSum);
        projectsStorageSum = new HashMap<>(other.projectsStorageSum);
    }

    public void bumpRevision() {
        this.revision++;
    }

    public void setRevision(long revision) {
        this.revision = revision;
    }
}
