#!/bin/bash

#Display an error and exit
errorExit() {
    echo -e "\033[31mERROR: $1 \033[0m"; echo
    exit 1
}
SCRIPT_DIR=$(dirname $0)
SCRIPT_HOME="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"

initHelper() {
    export PROJECT_BUILD_DIR="${1}"
    local yqVersion="${2}"
    local OS_TYPE=$(uname)
    export YQ_PATH="${SCRIPT_HOME}/target/yq-linux/yq-${yqVersion}-linux/yq"
    if [ "${OS_TYPE}" == "Darwin" ]; then
        export YQ_PATH="${SCRIPT_HOME}/target/yq-darwin/yq-${yqVersion}-darwin/yq"
    fi
    source "${SCRIPT_HOME}/target/configuration-manager/constansts.sh"
    PRODUCT_NAME=$ARTIFACTORY_LABEL
}

createTemplateDir() {
    [ -d "${PROJECT_BUILD_DIR}/target/templates" ] && rm -rf "${PROJECT_BUILD_DIR}/target/templates"
    mkdir -p "${PROJECT_BUILD_DIR}/target/templates" || errorExit "Failed to create ${PROJECT_BUILD_DIR}/target/templates"
}

createArtifactoryTemplate() {
    local flavour="${1}"
    local rtTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose.yaml"
    local originalRtTemplate="${SCRIPT_HOME}/templates/docker-compose.yaml"
    touch "${rtTemplate}"
    yamlMerge "${rtTemplate}" "${originalRtTemplate}"
    updateYamlValue "${rtTemplate}" "services.artifactory.image" "\${DOCKER_REGISTRY}/jfrog/artifactory-${flavour}:\${ARTIFACTORY_VERSION}"
}

createNginxTemplate() {
    local flavour="${1}"
    local rtTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose.yaml"
    local rtNginxTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose-nginx.yaml"
    local originalNginxTemplate="${SCRIPT_HOME}/templates/docker-compose-nginx.yaml"
    touch "${rtNginxTemplate}"
    cp -f "${rtTemplate}" "${rtNginxTemplate}"
    yamlMerge "${rtNginxTemplate}" "${originalNginxTemplate}"
}

createVolumeTemplate() {
    local flavour="${1}"
    local rtVolumeTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose-volumes.yaml"
    local originalVolumeTemplate="${SCRIPT_HOME}/templates/docker-compose-volumes.yaml"
    cp -f "${originalVolumeTemplate}" "${rtVolumeTemplate}"
    updateYamlValue "${rtVolumeTemplate}" "services.postgres.image" "\${DOCKER_REGISTRY}/postgres:${POSTGRES_VERSION}-alpine"
    updateYamlValue "${rtVolumeTemplate}" "services.artifactory.image" "\${DOCKER_REGISTRY}/jfrog/artifactory-${flavour}:\${ARTIFACTORY_VERSION}"
}

createPostgresTemplates() {
    setPostgresVersion
    local length=${#JFROG_POSTGRES_VERSIONS[@]}
    for (( i=0; i<$length; i++ )); do
        local ver=$(echo "${JFROG_POSTGRES_VERSIONS[i]}" | tr "." "-")
        local version="${JFROG_POSTGRES_VERSIONS[i]}"
        local postgresTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose-postgres-${ver}v.yaml"
        local originalPostgresTemplate="${SCRIPT_HOME}/templates/docker-compose-postgres.yaml"
        cp -f "${originalPostgresTemplate}" "${postgresTemplate}"
        if [[ "${JFROG_POSTGRES_VERSIONS[i]}" == "${JFROG_POSTGRES_VERSIONS[0]}" ]]; then
            updateYamlValue "${postgresTemplate}" "services.postgres.image" "\${DOCKER_REGISTRY}/postgres:${version}"
        else
            updateYamlValue "${postgresTemplate}" "services.postgres.image" "\${DOCKER_REGISTRY}/postgres:${version}-alpine"
        fi
        if [[ "${JFROG_POSTGRES_VERSIONS[i]}" == "${JFROG_POSTGRES_VERSIONS[${length}-1]}" ]]; then
            mv "${postgresTemplate}" "${PROJECT_BUILD_DIR}/target/templates/docker-compose-postgres.yaml"
        fi
    done
}

createMainTemplate() {
    local mainTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose-all.yaml"
    local rtNginxTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose-nginx.yaml"
    local postgresTemplate="${PROJECT_BUILD_DIR}/target/templates/docker-compose-postgres.yaml"
    cp -f "${rtNginxTemplate}" "${mainTemplate}"
    yamlMerge "${mainTemplate}" "${postgresTemplate}"
}

yamlMerge() {
    local initialYamlFile="${1}"
    local additionalYamlFile="${2}"
    ${YQ_PATH} m -i "${initialYamlFile}"  "${additionalYamlFile}" "${initialYamlFile}" > /dev/null || errorExit "Failed to merge ${additionalYamlFile} to  ${initialYamlFile}"
}

updateYamlValue() {
    local ymlFile="${1}"
    local path="${2}"
    local value="${3}"
    ${YQ_PATH} w -i "${ymlFile}" "${path}" "${value}" || errorExit "Failed to set value for the path"
}


initHelper "${1}" "${3}"
createTemplateDir
createArtifactoryTemplate "${2}"
createNginxTemplate "${2}"
createVolumeTemplate "${2}"
createPostgresTemplates
createMainTemplate