package org.artifactory.webapp.main;

import com.google.common.base.Charsets;
import com.jfrog.sysconf.test.SystemYamlBuilder;
import com.jfrog.sysconf.test.YamlNodeProps;
import com.jfrog.sysconf.test.YamlServiceSysConfig;
import com.jfrog.sysconf.test.YamlSysConfig;
import org.apache.catalina.startup.Tomcat;
import org.apache.commons.io.FileUtils;
import org.artifactory.webapp.WebappUtils;
import org.jfrog.common.YamlUtils;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.UUID;

import static org.artifactory.webapp.WebappUtils.copyNewerDevResources;
import static org.artifactory.webapp.WebappUtils.updateMimetypes;
import static org.artifactory.webapp.main.StartArtifactoryDev.setSystemProperties;
import static org.artifactory.webapp.main.StartArtifactoryDev.startAccessProcess;
import static org.artifactory.webapp.main.StartArtifactoryDev.startFrontendProcess;
import static org.artifactory.webapp.main.StartArtifactoryDev.startMetadataProcess;
import static org.artifactory.webapp.main.StartArtifactoryDev.startRouterProcess;
import static org.jfrog.config.home.JFrogHome.ETC_FOLDER;
import static org.jfrog.config.home.JFrogHome.SYSTEM_YAML_FILENAME;
import static org.jfrog.config.home.JFrogHome.VAR_FOLDER;

/**
 * @author Tamir Hadad
 */
public class StartArtifactoryDevWithServices {

    public static void main(String... args) throws Exception {
        File devArtHome = StartArtifactoryDev.getArtifactoryDevHome(args);

        File devEtcDir = WebappUtils.getDevEtcFolder();
        updateMimetypes(devEtcDir);
        copyNewerDevResources(devEtcDir, devArtHome, true);
        setSystemProperties();
        createDevSystemYaml(devArtHome);
        AccessProcess accessProcess = startAccessProcess(8040, false, devArtHome, 8080);
        startRouterProcess(accessProcess, devArtHome);
        startFrontendProcess(devArtHome);
        startMetadataProcess(devArtHome);
        Tomcat tomcat = StartArtifactoryDev.startTomcatWithRouter(devArtHome);
        tomcat.getServer().await();
    }

    @SuppressWarnings("ConstantConditions")
    private static void createDevSystemYaml(File home) {
        File systemYamlFile = new File(
                Paths.get(home.getAbsolutePath(), VAR_FOLDER, ETC_FOLDER, SYSTEM_YAML_FILENAME).toUri());
        YamlSysConfig systemYamlConfig;
        YamlNodeProps.Builder yamlNodePropsBuilder = YamlNodeProps.builder().id(UUID.randomUUID().toString());
        try {
            if (systemYamlFile.exists()) {
                YamlSysConfig yamlSysConfig = YamlUtils.getInstance().readValue(systemYamlFile, YamlSysConfig.class);
                systemYamlConfig = yamlSysConfig
                        .withShared(YamlServiceSysConfig.builder().node(yamlNodePropsBuilder.build()).build());
            } else {
                systemYamlConfig = YamlSysConfig.builder()
                        .shared(YamlServiceSysConfig.builder().node(yamlNodePropsBuilder.build()).build())
                        .build();
            }
            FileUtils.deleteQuietly(systemYamlFile);
            FileUtils.writeStringToFile(systemYamlFile, SystemYamlBuilder.toYaml(systemYamlConfig), Charsets.UTF_8,
                    false);
        } catch (IOException e) {
            throw new IllegalStateException("Unable to create/append to system.yaml.", e);
        }
    }
}
