package org.artifactory.storage.db;

import org.artifactory.aol.AolService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.storage.jobs.async.database.DatabaseConverterRunnable;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.version.ArtifactoryVersionProvider;
import org.artifactory.version.CompoundVersionDetails;
import org.jetbrains.annotations.NotNull;
import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

/**
 * @author Alexei Vainshtein
 */
public class AsyncDatabaseConverterAdapterTest extends ArtifactoryHomeBoundTest {

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testConvert() {
        ArtifactoryContext context = Mockito.mock(ArtifactoryContext.class);
        createAolService(context, false);
        AsyncDatabaseConverterAdapter converterAdapter =
                Mockito.spy(new AsyncDatabaseConverterAdapter(homeStub));
        CompoundVersionDetails source = Mockito.mock(CompoundVersionDetails.class);
        Mockito.when(source.getVersion()).thenReturn(ArtifactoryVersionProvider.v790m001.get());
        ScheduledExecutorService executor =
                Mockito.mock(TestArtifactoryExecutor.class, InvocationOnMock::callRealMethod);
        Mockito.when(converterAdapter.getScheduledExecutorService()).thenReturn(executor);
        converterAdapter.convert(source, null);
        Mockito.verify(executor, Mockito.times(1)).shutdown();
    }

    @Test
    public void testGetMinutesDelay() {
        AsyncDatabaseConverterAdapter converterAdapter = Mockito.spy(new AsyncDatabaseConverterAdapter(homeStub));
        ArtifactoryContext context = Mockito.mock(ArtifactoryContext.class);
        AolService aolService = createAolService(context, true);

        int minutesDelay = converterAdapter.getMinutesDelay();
        Assert.assertTrue(minutesDelay > 0);

        Mockito.when(aolService.isAol()).thenReturn(false);
        minutesDelay = converterAdapter.getMinutesDelay();
        Assert.assertEquals(minutesDelay, 0);
    }

    @NotNull
    private AolService createAolService(ArtifactoryContext context, boolean isAol) {
        AolService aolService = Mockito.mock(AolService.class);
        Mockito.when(aolService.isAol()).thenReturn(isAol);
        ArtifactoryContextThreadBinder.bind(context);
        Mockito.when(context.beanForType(AolService.class)).thenReturn(aolService);
        return aolService;
    }

    private static abstract class TestArtifactoryExecutor implements ScheduledExecutorService {

        @NotNull
        @Override
        public ScheduledFuture<?> schedule(@NotNull Runnable command, long delay,
                @NotNull TimeUnit unit) {
            Assert.assertTrue(command instanceof DatabaseConverterRunnable);
            return null;
        }

        @Override
        public void shutdown() {

        }
    }
}