package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.upgrades.common.AsyncUpgradeBaseTest;
import org.jfrog.storage.DbType;
import org.jfrog.storage.util.DbUtils;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.sql.SQLException;

/**
 * @author Alexei Vainshtein
 */
public class V225UpgradePostgreSqlNodesIndexesTest extends AsyncUpgradeBaseTest {

    @Test
    public void testNodesNodePath() throws SQLException {
        if (dbProperties.getDbType() != DbType.POSTGRESQL) {
            logger.info("This test is for Postgresql only, found " + dbProperties.getDbType() + ". Skipping test. ");
            return;
        }

        String indexDef = DbUtils.getPostgresqlIndexDefInfo(jdbcHelper, "nodes_node_path_idx");
        assertIndexDef("CREATE INDEX nodes_node_path_idx ON public.nodes USING btree (node_path varchar_pattern_ops)",
                indexDef);
    }

    @Test
    public void testNodesNodeNameIndex() throws SQLException {
        if (dbProperties.getDbType() != DbType.POSTGRESQL) {
            logger.info("This test is for Postgresql only, found " + dbProperties.getDbType() + ". Skipping test. ");
            return;
        }

        String indexDef = DbUtils.getPostgresqlIndexDefInfo(jdbcHelper, "nodes_node_name_idx");
        assertIndexDef("CREATE INDEX nodes_node_name_idx ON public.nodes USING btree (node_name varchar_pattern_ops)",
                indexDef);
    }

    @Test
    public void testNodesRepoPathIndex() throws SQLException {
        if (dbProperties.getDbType() != DbType.POSTGRESQL) {
            logger.info("This test is for Postgresql only, found " + dbProperties.getDbType() + ". Skipping test. ");
            return;
        }

        String indexDef = DbUtils.getPostgresqlIndexDefInfo(jdbcHelper, "nodes_repo_path_name_idx");
        assertIndexDef("CREATE UNIQUE INDEX nodes_repo_path_name_idx ON public.nodes USING btree (repo " +
                "varchar_pattern_ops, " +
                "node_path varchar_pattern_ops, node_name varchar_pattern_ops)", indexDef);
    }

    private void assertIndexDef(String index, String indexDef) {
        Assert.assertEquals(indexDef, index);
    }
}
