package org.artifactory.storage.db.version.converter;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.storage.db.event.service.NodeEventCursorService;
import org.artifactory.storage.db.util.JdbcHelper;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.storage.DbType;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.function.BiPredicate;

/**
 * @author Alexei Vainshtein
 */
public class AsyncDBSqlConditionalConverterTest extends ArtifactoryHomeBoundTest {

    private ArtifactoryContext context;

    @BeforeMethod
    public void setUp() {
        context = Mockito.mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
    }

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testConvertSuccess() throws SQLException {
        AsyncDBSqlConditionalConverter asyncDBSqlConditionalConverter = getAsyncDBSqlConverter();
        org.jfrog.storage.JdbcHelper jdbcHelper = Mockito.mock(org.jfrog.storage.JdbcHelper.class);
        ResultSet rs = Mockito.mock(ResultSet.class);
        Mockito.when(rs.next()).thenReturn(true);
        Mockito.when(rs.getString("indexdef")).thenReturn("check");
        Mockito.when(jdbcHelper.executeSelect(Mockito.anyString())).thenReturn(rs);
        asyncDBSqlConditionalConverter.convert(jdbcHelper, DbType.POSTGRESQL);
    }

    @Test
    public void testConvertFail() {
        AsyncDBSqlConditionalConverter asyncDBSqlConditionalConverter = getAsyncDBSqlConverter();
        JdbcHelper jdbcHelper = Mockito.mock(JdbcHelper.class);
        NodeEventCursorService service = Mockito.mock(NodeEventCursorService.class);
        Mockito.when(context.beanForType(NodeEventCursorService.class)).thenReturn(service);
        try {
            asyncDBSqlConditionalConverter.convert(jdbcHelper, DbType.POSTGRESQL);
        } catch (Exception e) {
            Assert.fail("No exceptions should be thrown.");
        }
        Mockito.verify(asyncDBSqlConditionalConverter, Mockito.times(3))
                .convertWithRetries(Mockito.eq(jdbcHelper), Mockito.eq(DbType.POSTGRESQL), Mockito.anyInt());
    }

    @Test
    public void testConvertWithRetry() {
        AsyncDBSqlConditionalConverter asyncDBSqlConditionalConverter = getAsyncDBSqlConverter();
        JdbcHelper jdbcHelper = Mockito.mock(JdbcHelper.class);
        try {
            asyncDBSqlConditionalConverter.convertWithRetries(jdbcHelper, DbType.POSTGRESQL, 0);
            Assert.fail("Expected the convert to fail");
        } catch (Exception e) {
            Assert.assertTrue(e instanceof RuntimeException);
        }
        Mockito.verify(asyncDBSqlConditionalConverter, Mockito.times(3))
                .convertWithRetries(Mockito.eq(jdbcHelper), Mockito.eq(DbType.POSTGRESQL), Mockito.anyInt());
    }

    private AsyncDBSqlConditionalConverter getAsyncDBSqlConverter() {
        ConversionPredicate conversionPredicate = Mockito.mock(ConversionPredicate.class);
        BiPredicate<org.jfrog.storage.JdbcHelper, DbType> biPredicate = Mockito.mock(BiPredicate.class);
        Mockito.when(conversionPredicate.condition()).thenReturn(biPredicate);
        Mockito.when(biPredicate.test(Mockito.any(), Mockito.any())).thenReturn(true);
        return Mockito.spy(new AsyncDBSqlConditionalConverter("v225_change_nodes_node_name_idx", conversionPredicate));
    }
}