package org.artifactory.ui.rest.service.artifacts.browse.treebrowser.tabs.permission;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.api.security.UserGroupService;
import org.artifactory.repo.RepoPath;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.ui.rest.service.builds.buildsinfo.tabs.permissions.GetBuildEffectivePermissionService;
import org.artifactory.ui.rest.service.distribution.GetReleaseBundleEffectivePermissionService;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;

/**
 * @author Yevdo Abramov
 * Created on 27/10/2020
 */
@Test
public class GetRepoEffectivePermissionServiceTest {
    @Mock
    private AuthorizationService authService;
    @Mock
    private RepositoryService repoService;
    @Mock
    private GetBuildEffectivePermissionService buildEffectivePermissionService;
    @Mock
    private GetReleaseBundleEffectivePermissionService releaseBundleEffectivePermissionService;
    @Mock
    private UserGroupService userGroupService;

    private GetRepoEffectivePermissionService service;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        service = new GetRepoEffectivePermissionService(authService, repoService, userGroupService,
                buildEffectivePermissionService, releaseBundleEffectivePermissionService);
    }

    @Test
    public void testExecuteWhenNoPermissions() {
        ArtifactoryRestRequest request = mock(ArtifactoryRestRequest.class);
        RestResponse response = mock(RestResponse.class);

        when(request.getQueryParamByKey("repoKey")).thenReturn("my_repo");
        when(request.getQueryParamByKey("path")).thenReturn("my_path");
        when(authService.canManage(any(RepoPath.class))).thenReturn(false);

        service.execute(request, response);

        verify(request).getQueryParamByKey("repoKey");
        verify(request).getQueryParamByKey("path");
        verify(authService).canManage(any());
        verifyNoMoreInteractions(userGroupService);
    }

    @Test
    public void testExecuteIsAcceptingManagePermissions() {
        ArtifactoryRestRequest request = mock(ArtifactoryRestRequest.class);
        RestResponse response = mock(RestResponse.class);

        when(request.getQueryParamByKey("repoKey")).thenReturn("my_repo");
        when(request.getQueryParamByKey("path")).thenReturn("my_path");
        when(authService.canManage(any(RepoPath.class))).thenReturn(true);

        service.execute(request, response);

        verify(request).getQueryParamByKey("repoKey");
        verify(request).getQueryParamByKey("path");
        verify(authService).canManage(any());
        verify(userGroupService).getRepoPathAcls(any());
    }
}