package org.artifactory.ui.rest.service.distribution;


import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.release.bundle.ReleaseArtifactInternalModel;
import org.artifactory.addon.release.bundle.ReleaseBundleAddon;
import org.artifactory.api.rest.distribution.bundle.models.BundleReleaseNotes;
import org.artifactory.api.rest.distribution.bundle.models.ReleaseArtifact;
import org.artifactory.api.rest.distribution.bundle.models.ReleaseBundleModel;
import org.artifactory.bundle.BundleType;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.ui.rest.model.distribution.ReleaseBundleIModel;
import org.codehaus.jackson.map.ObjectMapper;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;


public class GetReleaseBundleServiceTest extends ArtifactoryHomeBoundTest {

    private ObjectMapper mapper = new ObjectMapper();

    @Mock
    AddonsManager addonsManager;

    @BeforeMethod
    public void init() {
        MockitoAnnotations.initMocks(this);
    }

    @InjectMocks
    GetReleaseBundleService getReleaseBundleService = new GetReleaseBundleService();

    @Test
    public void testExecute() throws IOException {
        ArtifactoryRestRequest request = mock(ArtifactoryRestRequest.class);
        ReleaseBundleAddon releaseBundleAddon = mock(ReleaseBundleAddon.class);
        RestResponse response = new ArtifactoryRestResponse();

        when(addonsManager.addonByType(ReleaseBundleAddon.class)).thenReturn(releaseBundleAddon);
        when(request.getPathParamByKey("name")).thenReturn("RB");
        when(request.getPathParamByKey("version")).thenReturn("1.0");
        when(request.getPathParamByKey("type")).thenReturn("source");
        when(releaseBundleAddon.getBundleModel("RB", "1.0", BundleType.SOURCE)).
                thenReturn(createReleaseBundleModelForTest());
        when(releaseBundleAddon.getReleaseArtifactsUsingAql("RB",
                "1.0",
                BundleType.SOURCE)).thenReturn(List.of(createArtifactModel()));
        when(releaseBundleAddon.getStoringRepo("RB", "1.0", BundleType.SOURCE))
                .thenReturn("release-bundle");
        getReleaseBundleService.execute(request, response);
        ReleaseBundleIModel model = mapper.readValue((String) response.getIModel(), ReleaseBundleIModel.class);

        Assert.assertEquals(model.getName(), "RB");
        Assert.assertEquals(model.getVersion(), "1.0");
        Assert.assertEquals(model.getReleaseNotes().getContent(), "release notes");
    }

    private ReleaseArtifactInternalModel createArtifactModel() {
        ReleaseArtifactInternalModel resModel = new ReleaseArtifactInternalModel();
        resModel.setName("artifact");
        resModel.setPath("release-bundles");
        resModel.setSize(10);
        resModel.setCreatedBy("gavriel");
        return resModel;

    }

    private ReleaseBundleModel createReleaseBundleModelForTest() {
        ReleaseBundleModel resModel = new ReleaseBundleModel();
        resModel.setName("RB");
        resModel.setVersion("1.0");
        resModel.setDescription("description");
        resModel.setReleaseNotes(new BundleReleaseNotes("release notes", "plain_text"));
        resModel.setStatus("complete");
        resModel.setArtifacts(List.of(new ReleaseArtifact()));
        resModel.setStoringRepo("release-bundles");
        return resModel;
    }
}