package org.artifactory.rest.resource.system;

import org.apache.http.HttpStatus;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.ArtifactoryRunningMode;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.security.access.AccessService;
import org.artifactory.spring.InternalArtifactoryContext;
import org.artifactory.storage.StorageService;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import javax.ws.rs.core.Response;

import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class PingResourceTest extends ArtifactoryHomeBoundTest {

    private InternalArtifactoryContext context;

    @BeforeMethod
    public void setup() {
        context = mock(InternalArtifactoryContext.class);
        when(context.isOffline()).thenReturn(false);
        when(context.getArtifactoryHome()).thenReturn(homeStub);
        StorageService storageService = mock(StorageService.class);
        doNothing().when(storageService).ping();
        when(context.beanForType(StorageService.class)).thenReturn(storageService);
        AccessService accessService = mock(AccessService.class);
        doNothing().when(accessService).ping();
        when(context.beanForType(AccessService.class)).thenReturn(accessService);
        ArtifactoryContextThreadBinder.bind(context);
    }

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void pingArtifactoryFreeSubscription() {
        AddonsManager addonsManager = Mockito.mock(AddonsManager.class);
        when(addonsManager.lockdown()).thenReturn(false);
        Mockito.when(addonsManager.getArtifactoryRunningMode()).thenReturn(ArtifactoryRunningMode.HA);
        PingResource pingResource = new PingResource(addonsManager);
        Response response = pingResource.pingArtifactory(false);
        Assert.assertEquals(response.getStatus(), HttpStatus.SC_OK);
    }

    @Test
    public void pingArtifactoryNonFreeSubscription() {
        AddonsManager addonsManager = Mockito.mock(AddonsManager.class);
        when(addonsManager.lockdown()).thenReturn(true);
        Mockito.when(addonsManager.getArtifactoryRunningMode()).thenReturn(ArtifactoryRunningMode.HA);
        PingResource pingResource = new PingResource(addonsManager);
        Response response = pingResource.pingArtifactory(false);
        Assert.assertEquals(response.getStatus(), HttpStatus.SC_SERVICE_UNAVAILABLE);
        Assert.assertEquals((String) response.getEntity(), "Server state is offline");
    }
}